#include "msp430x22x4.h"

static struct st_bit                      // rbg`p
{
  unsigned char B0:1;
  unsigned char B1:1;
  unsigned char B2:1;
  unsigned char B3:1;
  unsigned char B4:1;
  unsigned char B5:1;
  unsigned char B6:1;
  unsigned char B7:1;
}st_bit;

#define P3OUTB (*(volatile struct st_bit *)0x0019) //P3OUT Address

#define POW P3OUTB.B0                     // 4053, Biasd
#define CAL P3OUTB.B1                     // CAL, COsensor ؑ

// Function protos
void Setup(void);                         // 
void Get_CO(void);                        // ZT[d̎擾
unsigned short Sample (void);             // ADϊ
void Alarm (char mode);                   // A[
unsigned int Cal_VLO(void);               // VLO̍Z
void t1ms(unsigned short dat);            // Basic Timer1ĝ5.4.2Ő

#define CO_sensor  1547.                  // COZT̊x nA/1000ppm
#define k_sensor   1.5/1023./100e3/CO_sensor/1e-12  // COZT̊xW

long ppm;                                 // COZx
unsigned int ad_buffer[4];                // buffer for 4 ADC samples
unsigned int ad_average;                  // Average 4 samples
unsigned int CAL_average;                 // Average 4 samples at CAL
unsigned int CO_average;                  // Average 4 samples at CO Sensor
unsigned char CO_cnt;                     // Counter for CO detections
unsigned char T_cnt;                      // Counter for Timer
unsigned short VLO_cnt;                   // Counter for VLO at 1s

void main(void)
{
  WDTCTL = WDTPW + WDTHOLD;               // Stop watchdog timer
  Setup();                                // Call init routine

  for (;;)                                // [v
  {
    LPM3;
    Get_CO();                             //@
    ppm = k_sensor * (CO_average - CAL_average);  // Zx̌vZ
    if (ppm > 150)
    {
      Alarm(2);                           // 150ppmȏȂA[[h2
    }
    else if (ppm > 50)
    {                                     // 50<Zx<150ppm̏ꍇB
      CO_cnt++;                           // CO_cnt 1Abv
      if (CO_cnt >=5 )                    // 5ȏȂA[[h1
      {
        Alarm(1);
      }
    }
    else
    {
      CO_cnt = 0;                         // COZx̂ŁACO_cntNA
    }
  }
}

void Get_CO(void)
{
  POW = 1;                                // 4053, Bias On
  CAL = 1;                                // ܂CAL

// OA0̐ݒ
  OA0CTL0 = OAN_1 + OAP_0 + OAPM_1 + OAADC1;    //
                                          // "-"[qA2/OA0I1ɐڑ
                                          // "+"[qA0/OA0I0ɐڑ
                                          // xSlow
                                          // o͒[qA1/OA0Oɐڑ
  OA0CTL1 = OAFC_0;                       // 샂[hGeneral-purpose opamp
  ADC10AE0 = 0x07;                        // P2.0`2AiO[qɐݒ

//TimerB̐ݒ(6s莞ԂŎgp)
  TBCTL = TBSSEL_1 + MC_1 + ID_3 + TBCLR; // TA = ACLK/8
  TBCCR0 = VLO_cnt * 6;                   // TB̃^C}[6sZbg
  TACCTL0 &= ~CCIE;                       // Disble CCR0 interrupt
  TBCCTL0 = CCIE;                         // Enable TBCCR0 interrupt

  LPM3;                                   // 6sLPM. TimerBŃEF[NAbv

  CAL_average = Sample();                 // CAL̓d

  CAL = 0;                                // CO
  LPM3;                                   // 6sLPM. TimerBŃEF[NAbv

  CO_average = Sample();                  // CO̓d

  POW = 0;                                // 4053, Bias Off
  OA0CTL0 = 0;                            // OA0~
  OA0CTL1 = 0;
  ADC10AE0 = 0;
  TBCTL = 0;                              // TimerB~
  TBCCTL0 &= ~CCIE;                       // Disable TBCCR0 interrupt
  TACCTL0 = CCIE;                         // Enable CCR0 interrupt
}


void Setup(void)
{
  unsigned int a, counts;
  signed long temp = 1000000;             // NbN1MHz

// Ports
  P1DIR  = 0xfe;                          // All P1 but P1.0 as outputs
  P1OUT = 1;                              // All P1 but P1.0 outputs low
  P1IE = BIT0;                            // Enable switch interrupt
  P1IES = BIT0;                           // H to L transition for interrupt
  P1REN = BIT0;                           // Enable internal pull-up
  P1IFG = 0;                              // Clear any pending IFGs
  P2DIR  = 0xff;                          // Output direction
  P2OUT = 0;                              // All P2 low
  P3DIR  = 0xff;                          // Output direction
  P3OUT = 0;                              // All P3 low
  P4DIR  = 0xff;                          // Output direction
  P4OUT = 0;                              // All P4 low

// Setup DCO and VLO
  BCSCTL1 = CALBC1_1MHZ;                  // Use 1Mhz cal data for DCO
  DCOCTL = CALDCO_1MHZ;                   // Use 1Mhz cal data for DCO
  BCSCTL3 = LFXT1S_2;                     // Use VLO for ACLK

  counts = Cal_VLO();                     // counts is # of VLO clocks in 1MHz

// setup TA for 8 second interrupt
  TACTL = TASSEL_1 + ID_3 + TACLR;        // TA = ACLK/8
  TACCTL0 = CCIE;                         // Enable CCR0 interrupt

// Divide 1Mhz by counts to get # of VLO counts in 1 second period
  a = 0;
  do {                                    // a = temp / counts
    temp -= counts;
    a++;
  } while (temp > 0);

  VLO_cnt = a;                            // VLO1s̃JEg
  TACCR0 = 12 * a;                        // TACCR0 period for 12 second

// Start timer
  TACTL |= MC_1;                          // Up mode

  _EINT();                                // ݋
}

unsigned int Cal_VLO (void)
{
  unsigned int First_Cap, counts;

  BCSCTL1 |= DIVA_3;                    // Divide ACLK by 8(萸xグ邽߂/8Ƃ)
  TACCTL2 = CM_1 + CCIS_1 + CAP;        // Capture on ACLK
  TACTL = TASSEL_2 + MC_2 + TACLR;      // Start TA, SMCLK(DCO), Continuous
  while ((TACCTL2 & CCIFG) == 0);       // Wait until capture

  TACCR2 = 0;                           // Ignore first capture
  TACCTL2 &= ~CCIFG;                    // Clear CCIFG

  while ((TACCTL2 & CCIFG) == 0);       // Wait for next capture

  First_Cap = TACCR2;                   // Save first capture
  TACCTL2 &= ~CCIFG;                    // Clear CCIFG

  while ((TACCTL2 & CCIFG) ==0);        // Wait for next capture

  counts = (TACCR2 - First_Cap);        // # of VLO clocks in 8Mhz
  BCSCTL1 &= ~DIVA_3;                   // Clear ACLK/8 settings
  return counts;
}

unsigned short Sample (void)
{
  unsigned volatile int i, temp;

  ADC10CTL0 = ADC10ON + REFON + ADC10SHT_1 + MSC + ADC10IE + SREF_1;
                                        // ADC on, ref = 1.5V, sampling = 8 clocks
                                        // NOTE: REF takes 30us to settle, But
                                        // because of other instructs
                                        // no need for additional delay loop
  ADC10CTL1 = INCH_1 + CONSEQ_2;        // Channel A4, repeat single channel mode
  ADC10DTC1 = 4;                        // Do 4 conversions
  ADC10SA = (unsigned int)ad_buffer;    // Start address for DTC

  ADC10CTL0 |= ENC + ADC10SC;           // Enable and start conversions
  LPM3;                                 // Enter LPM3 while 4 conversions made

  ADC10CTL0 &= ~ENC;                    // Clear ENC to stop conversion
  ADC10CTL0 = 0;                        // Turn off ADC and reference

  ad_average = 0;

  for (i = 0; i < 4; i++)
  {
    ad_average += ad_buffer[i];         // ADϊl̕ς߂
  }
  return ad_average >> 2;
}

void Alarm (char mode)
{
  unsigned volatile int i, temp;

  switch (mode)
  {
  case 0:
    // [h0̃A[vOLq
    break;
  case 1:
    // [h1̃A[vOLq
    break;
  case 2:
    // [h2̃A[vOLq
    break;
  case 3:
    // [h3̃A[vOLq
    break;
  case 4:
    // [h4̃A[vOLq
    break;
  }
}


// Timer A0 interrupt service routine
#pragma vector=TIMERA0_VECTOR
__interrupt void Timer_A0 (void)
{
  T_cnt++;
  if (T_cnt >= 10)                        // 12s~10120s̊
  {
    T_cnt = 0;
    LPM3_EXIT;
  }
}

// Timer B0 interrupt service routine
#pragma vector=TIMERB0_VECTOR
__interrupt void Timer_B0 (void)
{
  LPM3_EXIT;
}

// ADC10 interrupt service routine
#pragma vector=ADC10_VECTOR
__interrupt void ADC10 (void)
{
    LPM3_EXIT;
}


// P1.0 interrupt service routine
#pragma vector=PORT1_VECTOR
__interrupt void P1_ISR (void)
{
  unsigned volatile int i;
  i = 5000;                               // ~50ms Debounce delay
  do i--;                                 //
  while (i != 0);

  P1IFG = 0;                              // Clear IFG
  Alarm(0);                               // A[A[h0
}

/******************************************************
// dat(ms)Ԃԑ҂
******************************************************/
void t1ms(unsigned short dat)             // Basic Timer1ĝ5.4.2Ő
{
  unsigned long i;
  unsigned short j;

//  i = 32.768 * dat - 22;                  // ACLK(32.768kHz)o߃JEgvZ
  i = 4.096 / 3.33 * dat;                  // ACLK(32.768kHz)o߃JEgvZ
                                          // 22͎Ԓp(MCLK̒lŕς)
  do
  {
    j = TAR;                           // j͉ʂ̃JE^l
    while(TAR == j);                   // BTCNT1JEgAbv܂ő҂
    i--;                                  // iPJEg
  }while (i != 0);                        // i=0(^CAbv)܂ŌJԂ
}

